;
; Export file for creating MS DLL
;
; $Id: dcap.def,v 1.13 2004-02-26 15:19:27 tigran Exp $
;
LIBRARY dcap
DESCRIPTION "dCache client library, Win32 port"
EXPORTS

; POSIX IO

	dc_open
	dc_creat
	dc_close
	dc_close2
	dc_read
	dc_pread
	dc_pwrite
	dc_write
	dc_lseek
	dc_stat
	dc_lstat
	dc_fstat
	dc_fsync
	dc_dup
	dc_access

; libc streams

	dc_fopen
	dc_fdopen
	dc_fclose
	dc_fread
	dc_fwrite
	dc_fseek
	dc_feof
	dc_ftell
	dc_ferror
	dc_fflush


	dc_stage
	dc_check

; dcap specific functions

	dc_setOpenTimeout
	dc_setOnError
	dc_noBuffering
	dc_setBufferSize
	dc_unsafeWrite
	dc_setReplyHostName
	dc_setCallbackPort
	dc_setCallbackPortRange
	getDcapVersion
	dc_setTunnel
	dc_setDebugLevel
	dc_setStrDebugLevel
	dc_debug
	dc_perror
	dc_strerror
	dc_setExtraOption
	dc_setTCPSendBuffer
	dc_setTCPReceiveBuffer
	dc_errno DATA

; END
