/*
 * $Id: get_user.h,v 1.1 2002-10-23 10:33:25 cvs Exp $
 */


#ifndef GET_USER_H
#define GET_USER_H

#include <user_entry.h>

extern user_entry * getUserEntry();
extern void clear_entry( user_entry *);


#endif
