/*
 *   DCAP - dCache Access Protocol client interface
 *
 *   Copyright (C) 2000,2004 DESY Hamburg DMG-Division.
 *
 *   AUTHOR: Tigran Mkrtchayn (tigran.mkrtchyan@desy.de)
 *
 *   This program can be distributed under the terms of the GNU LGPL.
 *   See the file COPYING.LIB
 *
 */
 
 
/*
 * $Id: xutil.h,v 1.2 2004-11-01 19:33:30 tigran Exp $
 */
 
#ifndef _XUTIL_H
#define _XUTIL_H


extern char *xdirname(const char *);
extern char *xbasename(const char *);
extern char *xstrndup(const char *, size_t);



#endif /* _XUTIL_H */
