#ifndef DEBUG_MAP_H
#define DEBUG_MAP_H
/* This file generated by make procedure */

static debugMap debugMapArray[] = {
	{ 0,	"no_output"},
	{ 1,	"error"},
	{ 2,	"info"},
	{ 4,	"time"},
	{ 8,	"trace"},
	{ 16,	"calls"},
	{ 32,	"io"},
	{ 64,	"threads"},
	{ -1,	NULL}

};

#endif /* DEBUG_MAP_H */
